@@ -65,32 +65,71 @@ snd_mixer_selem_is_active ?
 #endif
 
 static snd_mixer_t	*handle = NULL;
-static snd_mixer_elem_t *elem = NULL;
 static char card[64] = "default";
 
+#define MAX_MASTERS 10
+
+static char const* master_ids[MAX_MASTERS] = {
+	"Master",
+	"PCM",
+	"Analog Front",
+	"Speaker",
+	NULL,
+};
+
+static void show_developer_hint(void)
+{
+	snd_mixer_elem_t*                first_control;
+	snd_mixer_elem_t*                last_control;
+	snd_mixer_elem_t*                current_control;
+	gboolean                         has_playback;
+	char const*                      control_name;
+	int                              control_index;
+
+	first_control = snd_mixer_first_elem (handle);
+	last_control = snd_mixer_last_elem (handle);
+
+	fprintf(stderr, _("alsa: error: master control not found. I even tried to guess wildly, but to no avail.\n"));
+	if (first_control == NULL) {
+		return;
+	}
+	
+	fprintf(stderr, _("alsa: info: developer information follows: (send E-Mail to Developer with that)\n"));
+
+	current_control = first_control;
+	while (current_control != NULL) {
+		control_name = snd_mixer_selem_get_name (current_control);
+		control_index = snd_mixer_selem_get_index (current_control);
+
+		has_playback = snd_mixer_selem_has_common_volume(current_control)
+		             ||  snd_mixer_selem_has_playback_volume(current_control);
+
+		fprintf(stderr, "alsa: * %s,%d", control_name, control_index);
+
+		if (has_playback) {
+			fprintf(stderr, "  <--- ");
+			fprintf(stderr, _("hint hint"));
+		}
+		fprintf(stderr, "\n");
+
+		current_control = snd_mixer_elem_next (current_control);
+	}
+	fprintf(stderr, _("alsa: info: end of developer information\n"));
+}
+
 static void find_master(void)
 {
-	int		err;
-	snd_mixer_selem_id_t	*sid, *sid2;
-	char buf[12] = "Master";
-	char buf2[12] = "PCM";
-	
-	elem = NULL;
+	int                     err;
+	int                     i;
 
-	snd_mixer_selem_id_alloca(&sid);
-	snd_mixer_selem_id_set_index(sid, 0);
-	snd_mixer_selem_id_set_name(sid, buf);
+	snd_mixer_selem_id_t*   master_selectors[MAX_MASTERS] = { NULL };
 
-	snd_mixer_selem_id_alloca(&sid2);
-	snd_mixer_selem_id_set_index(sid2, 0);
-	snd_mixer_selem_id_set_name(sid2, buf2);
-
-	if (handle) {
+	if (handle != NULL) {
 		snd_mixer_close(handle);
 		handle = NULL;
 	}
-	
-	if ((err = snd_mixer_open(&handle, 0)) < 0 || !handle) {
+
+	if ((err = snd_mixer_open(&handle, 0)) < 0 || handle == NULL) {
 		error(_("alsa: Mixer %s open error: %s\n"), card, snd_strerror(err));
 		return;
 	}
@@ -99,7 +138,6 @@ static void find_master(void)
 		error(_("alsa: Mixer attach %s error: %s\n"), card, snd_strerror(err));
 		snd_mixer_close(handle); /* <-- alsa 0.9.3a fails assert(hctl) if I do that... weeeeird... */
 		handle = NULL;
-		elem = NULL; /* just to be sure */
 		return;
 	}
 	if ((err = snd_mixer_selem_register(handle, NULL, NULL)) < 0) {
@@ -116,20 +154,9 @@ static void find_master(void)
 		error(_("alsa: Mixer load error: %s: %s\n"), card, snd_strerror(err));
 #endif
 		snd_mixer_close(handle);
+		handle = NULL;
 		return;
 	}
-	elem = snd_mixer_find_selem(handle, sid);
-	if (!elem) {
-		elem = snd_mixer_find_selem(handle, sid2);
-		if (!elem) {
-#ifdef DEBUG
-			error(_("alsa: Unable to find simple control '%s',%i\n"),
-			snd_mixer_selem_id_get_name(sid), snd_mixer_selem_id_get_index(sid));
-#endif
-			snd_mixer_close(handle);
-			return;
-		}
-	}
 }
 
 static void vc_set_device(char const *name)
@@ -151,7 +178,9 @@ static void vc_set_device(char const *name)
 static int vc_reinit_device(void)
 {
 	find_master();
-	if (!elem) return -1;
+	if (handle == NULL) {
+		return -1;
+	}
 	
 	return 0;
 }
@@ -205,75 +234,136 @@ static snd_mixer_elem_t * find_control(char const *which)
 
 static int vc_get_volume(char const *which)
 {
-	long pmin,pmax;
-	long lval;
-	int pb;
+	int playback_enable;
+	int capture_enable;
+	long playback_min;
+	long playback_max;
+	long capture_min;
+	long capture_max;
+	long playback_value;
+	long capture_value;
+
 	snd_mixer_selem_channel_id_t chn;
-	snd_mixer_elem_t *xelem = NULL;
+	snd_mixer_elem_t *xelem;
 
-	if (!handle) return 0;
-
-	if (which) {
-		xelem = find_control (which);
-	} else {
-		xelem = elem;
+	if (handle == NULL) {
+		return 0;
 	}
 
-	if (!xelem) return 0;
+	xelem = (which != NULL) ? find_control (which) : NULL;
 
-	snd_mixer_selem_get_playback_volume_range(xelem, &pmin, &pmax);
-	pmin = 0;
+	if (xelem == NULL) {
+		return 0;
+	}
+
+	snd_mixer_selem_get_playback_volume_range (xelem, &playback_min, &playback_max);
+	snd_mixer_selem_get_capture_volume_range (xelem, &capture_min, &capture_max);
+
+	playback_min = 0;
+	capture_min = 0;
 
 	/* if (snd_mixer_selem_has_playback_volume(xelem)) { */
 	for (chn = 0; chn <= SND_MIXER_SCHN_LAST; chn++) {
-		if (!snd_mixer_selem_has_playback_channel(xelem, chn)) continue;
-	
-		snd_mixer_selem_get_playback_volume(xelem, chn, &lval); 
+		if (!snd_mixer_selem_has_playback_channel(xelem, chn)) {
+			continue;
+		}
 
-		pb = TRUE;
-		snd_mixer_selem_get_playback_switch (xelem, chn, &pb);
-		if (!pb) {
+		snd_mixer_selem_get_playback_volume(xelem, chn, &playback_value); 
+
+		playback_enable = TRUE;
+		snd_mixer_selem_get_playback_switch (xelem, chn, &playback_enable);
+		if (playback_enable == 0) {
 			return 0;
 		}
 
 		/*error("%ld,%ld,%ld,%ld", pmin,pmax,lval,(lval - pmin) * 100 / (pmax-pmin));*/
-		if (pmax > pmin) {
-			return (lval - pmin) * 100 / (pmax-pmin);
+		if (playback_max > playback_min) {
+			return (playback_value - playback_min) * 100 / (playback_max - playback_min);
 		} else {
-			return lval;
+			return playback_value;
 		}
 	}
+
+	/* if this point is reached, only capture setting remain, so use those */
+	for (chn = 0; chn <= SND_MIXER_SCHN_LAST; chn++) {
+		if (!snd_mixer_selem_has_capture_channel(xelem, chn)) {
+			continue;
+		}
+
+		snd_mixer_selem_get_capture_volume(xelem, chn, &capture_value); 
+
+		capture_enable = TRUE;
+		snd_mixer_selem_get_capture_switch (xelem, chn, &capture_enable);
+		if (capture_enable == 0) {
+			return 0;
+		}
+
+		/*error("%ld,%ld,%ld,%ld", pmin,pmax,lval,(lval - pmin) * 100 / (pmax-pmin));*/
+		if (capture_max > capture_min) {
+			return (capture_value - capture_min) * 100 / (capture_max - capture_min);
+		} else {
+			return capture_value;
+		}
+	}
+
 	return 0;
 }
 
 static void vc_set_volume(char const *which, int vol_p)
 {
-	long pmin,pmax;
-	long lval;
-	snd_mixer_selem_channel_id_t chn;
-	snd_mixer_elem_t *xelem = NULL;
-	double vold;
-	
-	if (!handle) return;
-	
-	if (which) {
-		xelem = find_control (which);
-	} else {
-		xelem = elem;
-	}
-	
-	if (!xelem) return;
+	long playback_min;
+	long playback_max;
+	long capture_min;
+	long capture_max;
+	long playback_value;
+	long capture_value;
+	double playback_value_double;
+	double capture_value_double;
 
-	snd_mixer_selem_get_playback_volume_range(xelem, &pmin, &pmax);
-	pmin = 0;
+	/*snd_mixer_selem_channel_id_t chn;*/
+	snd_mixer_elem_t *xelem;
+	
+	if (handle == NULL) {
+		return;
+	}
+
+	xelem = (which != NULL) ? find_control (which) : NULL;
+	
+	if (xelem == NULL) {
+		return;
+	}
+
+	snd_mixer_selem_get_playback_volume_range (xelem, &playback_min, &playback_max);
+	snd_mixer_selem_get_capture_volume_range (xelem, &capture_min, &capture_max);
+
+	playback_min = 0;
+	capture_min = 0;
 
 	/*vol_p = (lval - pmin) * 100 / (pmax - pmin);*/
-	vold = vol_p;
-	if (vol_p != 0)
-		vold += 0.999999;
+	playback_value_double = vol_p;
+	capture_value_double = vol_p;
+	if (vol_p != 0) {
+		playback_value_double += 0.999999;
+		capture_value_double += 0.999999;
+	}
 		
-	lval = (long) pmin + (vold) * (pmax - pmin) / 100;
+	playback_value = (long) playback_min + (playback_value_double) * (playback_max - playback_min) / 100;
+	capture_value = (long) capture_min + (capture_value_double) * (capture_max - capture_min) / 100;
 
+	/*
+	snd_mixer_selem_get_playback_switch(Ex_elem, SND_MIXER_SCHN_MONO, &status);
+	*/
+	
+	/* use _all functions for sami's card ... */
+	if (playback_value == 0) /* mute */
+		snd_mixer_selem_set_playback_switch_all (xelem, 0);
+	else /* unmute, just in case. */
+		snd_mixer_selem_set_playback_switch_all (xelem, 1);
+		
+	snd_mixer_selem_set_playback_volume_all (xelem, playback_value);
+	snd_mixer_selem_set_capture_volume_all (xelem, capture_value);
+
+#if 0		
 	for (chn = 0; chn <= SND_MIXER_SCHN_LAST; chn++) {
 		if (!snd_mixer_selem_has_playback_channel(xelem, chn)) continue;
 
@@ -285,6 +375,7 @@ static void vc_set_volume(char const *which, int vol_p)
 			snd_mixer_selem_set_playback_volume(xelem, chn, lval);
 		}
 	}
+#endif
 }
 
 GList *alsa_enum_to_glist(snd_mixer_elem_t *i)
@@ -293,6 +384,7 @@ GList *alsa_enum_to_glist(snd_mixer_elem_t *i)
 	gint j;
 	char tmp[257];
 	gint cnt;
+	volchoice_t* choice;
 	
 	g = NULL;
 	if (!snd_mixer_selem_is_enumerated (i))
@@ -303,7 +395,11 @@ GList *alsa_enum_to_glist(snd_mixer_elem_t *i)
 		g_snprintf (tmp, 256, "%d", j);
 		snd_mixer_selem_get_enum_item_name (i, j, 256, tmp);
 		
-		g = g_list_append (g, g_strdup (tmp));
+		choice = g_new0 (volchoice_t, 1);
+		choice->name = g_strdup (tmp);
+		choice->displayname = g_strdup (tmp);
+		
+		g = g_list_append (g, choice);
 	}
 			
 	return g;
@@ -328,7 +424,7 @@ static GList *vc_get_control_list(void)
 	if (!g) return NULL;
 #endif
 
-	if (!handle) {
+	if (handle == NULL) {
 		return NULL;
 	}
 
@@ -384,7 +480,7 @@ static int alsa_cb(snd_mixer_t *ctl, unsigned int mask, snd_mixer_elem_t *elem)
 {
 	const char *which;
 	g_warning ("alsa_cb\n");
-	if (elem) {
+	if (elem && snd_mixer_elem_get_type (elem) == SND_MIXER_ELEM_SIMPLE) {
 		which = snd_mixer_selem_get_name (elem);
 	} else {
 		which = NULL;
@@ -401,7 +497,9 @@ static void vc_set_volume_callback(volchanger_callback_t cb, void *data)
 {
 	/* supports 1 (ONE) callback */
 
-	if (!handle) return;
+	if (handle == NULL) {
+		return;
+	}
 
 	mycb = cb;
 	mydata = data;
@@ -412,7 +510,9 @@ static void vc_set_volume_callback(volchanger_callback_t cb, void *data)
 
 static void vc_close_device()
 {
-	if (!handle) return;
+	if (handle == NULL) {
+		return;
+	}
 	
 	snd_mixer_close (handle); /* FIXME does this close all related stuff? */
 	handle = NULL;
@@ -490,15 +590,16 @@ static void vc_set_select(char const *which, gchar const *v)
 	gint i;
 	gchar *s;
 	snd_mixer_elem_t *xelem = NULL;
-	if (!handle) return;
-	if (which) {
-		xelem = find_control (which);
-	} else {
-		xelem = elem;
+
+	if (handle == NULL) {
+		return;
 	}
 	
-	if (!xelem) return;
+	xelem = (which != NULL) ? find_control (which) : NULL;
 	
+	if (xelem == NULL) {
+		return;
+	}
 	
 	g = vc_get_choices (xelem);
 	if (!g || !v)
@@ -523,14 +624,15 @@ static gchar *vc_get_select(char const *which)
 	GList *g;
 	gchar *s;
 	snd_mixer_elem_t *xelem = NULL;
-	if (!handle) return NULL;
-	if (which) {
-		xelem = find_control (which);
-	} else {
-		xelem = elem;
+	if (handle == NULL) {
+		return NULL;
 	}
+
+	xelem = (which != NULL) ? find_control (which) : NULL;
 	
-	if (!xelem) return NULL;
+	if (xelem == NULL) {
+		return NULL;
+	}
 
 	if (snd_mixer_selem_get_enum_item(xelem, SND_MIXER_SCHN_MONO, &jj) < 0)
 		return NULL;
@@ -554,14 +656,16 @@ static gchar *vc_get_select(char const *which)
 static void vc_set_switch(char const *which, gboolean v)
 {
 	snd_mixer_elem_t *xelem = NULL;
-	if (!handle) return;
-	if (which) {
-		xelem = find_control (which);
-	} else {
-		xelem = elem;
+
+	if (handle == NULL) {
+		return;
 	}
+
+	xelem = (which != NULL) ? find_control (which) : NULL;
 	
-	if (!xelem) return;
+	if (xelem == NULL) {
+		return;
+	}
 
 /*			&& (snd_mixer_selem_has_common_switch(i)
 			 || snd_mixer_selem_has_playback_switch(i)
@@ -579,13 +683,11 @@ static gboolean vc_get_switch(char const *which)
 	
 	if (!handle) return FALSE;
 	
-	if (which) {
-		xelem = find_control (which);
-	} else {
-		xelem = elem;
-	}
+	xelem = (which != NULL) ? find_control (which) : NULL;
 	
-	if (!xelem) return FALSE;
+	if (xelem == NULL) {
+		return FALSE;
+	}
 
 	chn = SND_MIXER_SCHN_MONO;
 
